@@ -31,6 +31,7 @@ pstudent <- function(q, df = stop("df is required"), mu = 0, sigma = 1,
 
 qstudent <-  function(p, df = stop("df is required"), mu = 0, sigma = 1) {
   # quantiles of student's distribution
+  #
   # Args:
   #  p: the probabilities to find quantiles for
   #  df: degrees of freedom
@@ -50,41 +51,134 @@ rstudent <-  function(n, df = stop("df is required"), mu = 0, sigma = 1) {
   mu + sigma * rt(n, df = df)
 }
 
-dmultinormal <- function(x, mu, Sigma, log = TRUE) {
-  # density of the multinormal distribution
+dmulti_normal <- function(x, mu, Sigma, log = TRUE,
+                         check = FALSE) {
+  # density of the multivariate normal distribution 
+  # not vectorized to increase speed when x is only a vector not a matrix
   #
   # Args:
   #   x: the value(s) at which the density should be evaluated
   #   mu: mean vector
   #   sigma: covariance matrix
   #   log: return on log scale?
-  k <- length(x)
-  rooti <- backsolve(chol(Sigma), diag(k))
+  #   check: check arguments for validity?
+  #
+  # Returns:
+  #   density of the multi_normal distribution a values x
+  p <- length(x)
+  if (check) {
+    if (length(mu) != p) {
+      stop("dimension of mu is incompatible")
+    }
+    if (!all(dim(Sigma) == c(p, p))) {
+      stop("dimension of Sigma is incompatible")
+    }
+    if (!isSymmetric(Sigma, tol = sqrt(.Machine$double.eps), 
+                     check.attributes = FALSE)) {
+      stop("Sigma must be a symmetric matrix")
+    }
+  }
+  rooti <- backsolve(chol(Sigma), diag(p))
   quads <- colSums((crossprod(rooti, (x - mu)))^2)
-  out <- -(k / 2) * log(2 * pi) + sum(log(diag(rooti))) - .5 * quads
+  out <- -(p / 2) * log(2 * pi) + sum(log(diag(rooti))) - .5 * quads
   if (!log) 
     out <- exp(out)
   out
 }
 
-rmultinormal <- function(n, mu, Sigma, check = FALSE) {
-  # random values of the multinormal distribution 
+rmulti_normal <- function(n, mu, Sigma, check = FALSE) {
+  # random values of the multivariate normal distribution 
   #
   # Args:
   #   n: number of random values
   #   mu: mean vector
   #   sigma: covariance matrix
-  #   check: check sigma for symmetry?
+  #   check: check arguments for validity?
+  #
+  # Returns:
+  #   n samples of multi_normal distribution of dimension length(mu) 
   p <- length(mu)
   if (check) {
-    if (!all(dim(Sigma) == c(p, p))) 
-      stop("incompatible arguments")
-    if (!isSymmetric(unname(Sigma)))
-      stop("Sigma is not symmetric")
+    if (!(is.wholenumber(n) && n > 0)) {
+      stop("n must be a positive integer")
+    }
+    if (!all(dim(Sigma) == c(p, p))) {
+      stop("dimension of Sigma is incompatible")
+    }
+    if (!isSymmetric(Sigma, tol = sqrt(.Machine$double.eps), 
+                     check.attributes = FALSE)) {
+      stop("Sigma must be a symmetric matrix")
+    }
   }
-  cholSigma <- chol(Sigma)
   samples <- matrix(rnorm(n * p), nrow = n, ncol = p)
-  mu + samples %*% cholSigma
+  mu + samples %*% chol(Sigma)
+}
+
+dmulti_student <- function(x, df, mu, Sigma, log = TRUE,
+                          check = FALSE) {
+  # density of the multivariate student-t distribution 
+  #
+  # Args:
+  #   x: the value(s) at which the density should be evaluated
+  #   df: degrees of freedom
+  #   mu: mean vector
+  #   sigma: covariance matrix
+  #   log: return on log scale?
+  #   check: check arguments for validity?
+  #
+  # Returns:
+  #   density of the multi_student distribution a values x
+  if (is.vector(x)) {
+    x <- matrix(x, ncol = length(x))
+  }
+  p <- ncol(x)
+  if (check) {
+    if (df <= 0) {
+      stop("df must be greater zero")
+    }
+    if (length(mu) != p) {
+      stop("dimension of mu is incompatible")
+    }
+    if (!all(dim(Sigma) == c(p, p))) {
+      stop("dimension of Sigma is incompatible")
+    }
+    if (!isSymmetric(Sigma, tol = sqrt(.Machine$double.eps), 
+                     check.attributes = FALSE)) {
+      stop("Sigma must be a symmetric matrix")
+    }
+  }
+  chol_Sigma <- chol(Sigma)
+  rooti <- backsolve(chol_Sigma, t(x) - mu, transpose = TRUE)
+  quads <- colSums(rooti^2)
+  out <- lgamma((p + df)/2) - (lgamma(df / 2) + sum(log(diag(chol_Sigma))) + 
+         p / 2 * log(pi * df)) - 0.5 * (df + p) * log1p(quads / df)
+  if (!log) 
+    out <- exp(out)
+  out
+}
+
+rmulti_student <- function(n, df, mu, Sigma, log = TRUE, 
+                          check = FALSE) {
+  # random values of the multivariate student-t distribution 
+  #
+  # Args:
+  #   n: number of random values
+  #   df: degrees of freedom
+  #   mu: mean vector
+  #   sigma: covariance matrix
+  #   check: check arguments for validity?
+  #
+  # Returns:
+  #   n samples of multi_student distribution of dimension length(mu) 
+  p <- length(mu)
+  if (check) {
+    if (df <= 0) {
+      stop("df must be greater zero")
+    }
+  }
+  samples <- rmulti_normal(n, mu = rep(0, p), Sigma = Sigma, check = check) / 
+               sqrt(rchisq(n, df = df) / df)
+  sweep(samples, 2, mu, "+")
 }
 
 dcategorical <- function(x, eta, ncat, link = "logit") {
@@ -92,9 +186,12 @@ dcategorical <- function(x, eta, ncat, link = "logit") {
   # 
   # Args:
   #   x: positive integers not greater than ncat
-  #   mu: the linear predictor (of length or ncol ncat-1)  
+  #   eta: the linear predictor (of length or ncol ncat-1)  
   #   ncat: the number of categories
   #   link: the link function
+  #
+  # Returns:
+  #   probabilities P(X = x)
   if (is.null(dim(eta))) 
     eta <- matrix(eta, nrow = 1)
   if (length(dim(eta)) != 2 || !is.numeric(eta)) 
@@ -115,11 +212,12 @@ pcategorical <- function(q, eta, ncat, link = "logit") {
   #
   # Args:
   #   q: positive integers not greater than ncat
-  #   mu: the linear predictor (of length or ncol ncat-1)  
+  #   eta: the linear predictor (of length or ncol ncat-1)  
   #   ncat: the number of categories
+  #   link: a character string naming the link
   #
   # Retruns: 
-  #   probabilites P(x <= q)
+  #   probabilities P(x <= q)
   p <- dcategorical(1:max(q), eta = eta, ncat = ncat, link = link)
   do.call(cbind, lapply(q, function(j) rowSums(as.matrix(p[, 1:j]))))
 }
@@ -209,8 +307,10 @@ pordinal <- function(q, eta, ncat, family, link = "logit") {
   #
   # Args:
   #   q: positive integers not greater than ncat
-  #   mu: the linear predictor (of length or ncol ncat-1)  
+  #   eta: the linear predictor (of length or ncol ncat-1)  
   #   ncat: the number of categories
+  #   family: a character string naming the family
+  #   link: a character string naming the link
   #
   # Returns: 
   #   probabilites P(x <= q)
